@@ -9,11 +9,16 @@ from pragma_sdk.common.fetchers.fetchers.indexcoop import IndexCoopFetcher
 from pragma_sdk.common.fetchers.fetchers.kucoin import KucoinFetcher
 from pragma_sdk.common.fetchers.fetchers.okx import OkxFetcher
 from pragma_sdk.common.fetchers.fetchers.ekubo import EkuboFetcher
+from pragma_sdk.common.fetchers.fetchers.chainlink import ChainlinkFetcher
+from pragma_sdk.common.fetchers.fetchers.redstone import RedstoneFetcher
+from pragma_sdk.common.fetchers.fetchers.re7onchain import Re7OnChainFetcher
 from pragma_sdk.common.fetchers.fetchers.mexc import MEXCFetcher
 from pragma_sdk.common.fetchers.fetchers.gateio import GateioFetcher
 from pragma_sdk.common.fetchers.fetchers.dexscreener import DexscreenerFetcher
 from pragma_sdk.common.fetchers.fetchers.pyth import PythFetcher
 from pragma_sdk.common.fetchers.fetchers.upbit import UpbitFetcher
+from pragma_sdk.common.fetchers.fetchers.bitget import BitgetFetcher
+from pragma_sdk.common.fetchers.fetchers.lbank import LbankFetcher
 
 __all__ = [
     "BinanceFetcher",
@@ -32,4 +37,9 @@ __all__ = [
     "DexscreenerFetcher",
     "PythFetcher",
     "UpbitFetcher",
+    "BitgetFetcher",
+    "LbankFetcher",
+    "ChainlinkFetcher",
+    "RedstoneFetcher",
+    "Re7OnChainFetcher",
 ]
